@@ -28,6 +28,7 @@
 using namespace std;
 
 //inline void *aligned_calloc(size_t nmemb, size_t size) { return (void*)((unsigned)(calloc((nmemb*size)+15,sizeof(char)))+15 & 0xfffffff0); }
+inline void *aligned_calloc(size_t nmemb, size_t size) { return (void*)((size_t)(calloc((nmemb*size)+15,sizeof(char)))+15 & ~15); }
 
 // g++ -O3 -lm -ljack `gtk-config --cflags --libs` ex2.cpp
  
@@ -55,7 +56,7 @@ inline int 		int2pow2 (int x)	{ int r=0; while ((1<<r)<x) r++; return r; }
 *******************************************************************************
 *******************************************************************************/
 
-inline void *aligned_calloc(size_t nmemb, size_t size) { return (void*)((unsigned)(calloc((nmemb*size)+15,sizeof(char)))+15 & 0xfffffff0); }
+//inline void *aligned_calloc(size_t nmemb, size_t size) { return (void*)((unsigned)(calloc((nmemb*size)+15,sizeof(char)))+15 & 0xfffffff0); }
 
 
 <<includeIntrinsic>>
@@ -624,6 +625,7 @@ void GTKUI::addVerticalSlider(char* label, float* zone, float init, float min, f
 	gtk_signal_connect (GTK_OBJECT (adj), "value-changed", GTK_SIGNAL_FUNC (uiAdjustment::changed), (gpointer) c);
 	
 	GtkWidget* slider = gtk_vscale_new (GTK_ADJUSTMENT(adj));
+	gtk_range_set_inverted (GTK_RANGE(slider), TRUE);
 	gtk_scale_set_digits(GTK_SCALE(slider), precision(step));
 	gtk_widget_set_usize(slider, -1, 160);
 	
@@ -703,6 +705,7 @@ void GTKUI::addVerticalBargraph(char* label, float* zone, float lo, float hi)
 {
 	GtkWidget* pb = gtk_progress_bar_new();
 	gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(pb), GTK_PROGRESS_BOTTOM_TO_TOP);
+	gtk_widget_set_size_request(pb, 8, -1);
 	new uiBargraph(this, zone, GTK_PROGRESS_BAR(pb), lo, hi);
 	openFrameBox(label);
 	addWidget(label, pb);
@@ -714,6 +717,7 @@ void GTKUI::addHorizontalBargraph(char* label, float* zone, float lo, float hi)
 {
 	GtkWidget* pb = gtk_progress_bar_new();
 	gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(pb), GTK_PROGRESS_LEFT_TO_RIGHT);
+	gtk_widget_set_size_request(pb, -1, 8);
 	new uiBargraph(this, zone, GTK_PROGRESS_BAR(pb), lo, hi);
 	openFrameBox(label);
 	addWidget(label, pb);
@@ -827,7 +831,7 @@ void GTKUI::run()
 	assert(fTop == 0);
 	gtk_widget_show  (fBox[0]);
 	gtk_widget_show  (fWindow);
-	gtk_timeout_add(5, callUpdateAllGuis, 0);
+	gtk_timeout_add(40, callUpdateAllGuis, 0);
 	gtk_main ();
 	stop();
 }
